@@ -1,8 +1,8 @@
 import struct
 import socket
 import ipaddress
-import subprocess
 from ctypes import create_string_buffer, addressof
+from pcap_wrapper import bpf
 
 SO_ATTACH_FILTER = 26
 ETH_P_IP = 0x0800    # Internet Protocol packet
@@ -14,25 +14,17 @@ def get_s_g_bpf_filter_code(source, group, interface_name):
     ip_source_version = ipaddress.ip_address(source).version
     ip_group_version = ipaddress.ip_address(group).version
     if ip_source_version == ip_group_version == 4:
-        # cmd = "tcpdump -ddd \"(udp or icmp) and host %s and dst %s\"" % (source, group)
-        cmd = "tcpdump -ddd \"(ip proto not 2) and host %s and dst %s\"" % (source, group)
+        # bpf_filter_str = "(udp or icmp) and host %s and dst %s" % (source, group)
+        bpf_filter_str = "(ip proto not 2) and host %s and dst %s" % (source, group)
         protocol = ETH_P_IP
     elif ip_source_version == ip_group_version == 6:
         # TODO: allow ICMPv6 echo request/echo response to be considered multicast packets
-        cmd = "tcpdump -ddd \"(ip6 proto not 58) and host %s and dst %s\"" % (source, group)
+        bpf_filter_str = "(ip6 proto not 58) and host %s and dst %s" % (source, group)
         protocol = ETH_P_IPV6
     else:
         raise Exception("Unknown IP family")
 
-    result = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
-    bpf_filter = b''
-
-    tmp = result.stdout.read().splitlines()
-    num = int(tmp[0])
-    for line in tmp[1:]:
-        print(line)
-        bpf_filter += struct.pack("HBBI", *tuple(map(int, line.split(b' '))))
-
+    num, bpf_filter = bpf(bpf_filter_str.encode()).compiled_filter()
     print(num)
 
     # defined in linux/filter.h.
